/*=========================================================
       Device for switch ON/OFF HDD in laptop
	   
 Version: 1.0
 Date   : 13.01.2019
 (C) Barmaley, mculab@mail.ru
=========================================================*/

#include "iotiny12v.h"
#include "useful.h"

//   
#define KEY_SW_PORT       PORTB
#define KEY_SW_DIR        DDRB
#define KEY_SW_PIN        PINB
#define KEY_SW            BIT4

//    
#define POWER_SW_PORT     PORTB
#define POWER_SW_DIR      DDRB
#define POWER_SW          BIT3

//   
#define LED_PORT          PORTB
#define LED_DIR           DDRB
#define LED               BIT2

//   
//       100 
#define DELAY_KEY         0x1FFF

//    
#define TIME_KEY_MAX      0xFFFF
//      HDD
#define TIME_KEY_HDDON    0x000F
//        HDD
#define TIME_KEY_AHDDON   0x001F
//    
#define TIME_COUNT_DELAY  0x3FFF
//     EEPROM
#define TIME_EEPROM       0x00FF

//     EEPROM
#define MODE_START_ADR    0x00
#define MODE_AS_BYTE1     0x88
#define MODE_AS_BYTE2     0x11
#define MODE_AS           0x99

//   
#define DELAY_LED_OFF     0xFFFF
#define DELAY_LED_ON      0xFFFF
#define DELAY_LED_OFF_SM  DELAY_LED_OFF/5
#define DELAY_LED_ON_SM   DELAY_LED_ON/5


//   
void Delay(uint16 n)
{
  uint16 i;
  
  for( i=n; i; --i);
}

//    EEPROM
void writeE(uint8 adr, uint8 data)
{
   while (EECR & BIT(EEWE))		// Wait until EEWE flag is cleared
      ; // Empty

   EEAR = adr;			       // Load address
   EEDR = data;			       // Load data

   EECR = BIT(EEMWE);	       // EEPROM write enable
   EECR |= BIT(EEWE);
   //Delay(TIME_EEPROM);
}

//    EEPROM
uint8 readE(uint8 adr)
{
   EEAR = adr;			       // Load address
   EECR |= 0x01;		       // EEPROM read enable
   //Delay(TIME_EEPROM);
   
   return EEDR;			       // Return data
}



void main()
{

    uint16 TimerCount;
	uint8  Mode;
	uint8  tmp;
	
	//writeE(   MODE_START_ADR, 0);
	//writeE( MODE_START_ADR+1, 0);
	
	//      
    //  p-  
	POWER_SW_DIR  |=  ( POWER_SW );
	POWER_SW_PORT |=  ( POWER_SW );

    //   
	LED_DIR  |= ( LED );
	LED_PORT |= ( LED );

    //    
	KEY_SW_DIR  &= ~( KEY_SW );
	KEY_SW_PORT |=  ( KEY_SW );
	
	
	//    EEPROM   
	//  HDD   ,   
	tmp = readE( MODE_START_ADR );
	Mode = tmp + readE( MODE_START_ADR+1 );
	if ( Mode == MODE_AS ) POWER_SW_PORT &= ~( POWER_SW );
	//  ,        HDD
	tmp = 0;

	//  
    while(1)
      {
	    
		if( (KEY_SW_PIN & ( KEY_SW )) == 0 )
		  {
		    //       
			LED_PORT |= ( LED );
			Delay( DELAY_KEY );
			
			//    
			TimerCount = 0;
			while ( (KEY_SW_PIN & ( KEY_SW )) == 0 ) 
			  {
			    if ( TimerCount != TIME_KEY_MAX ) 
				  {
				    TimerCount++;
				    Delay( TIME_COUNT_DELAY );
			      }
				
				if ( TimerCount > TIME_KEY_HDDON )
			      LED_PORT &= ~( LED );
			      
				
				if ( TimerCount > TIME_KEY_AHDDON )
				  LED_PORT |= ( LED );
			  }
			  
			//         
			Delay( DELAY_KEY );
			  
			//       
			//   HDD
			if ( (TimerCount > TIME_KEY_HDDON) && (TimerCount < TIME_KEY_AHDDON) )
			  {
    	        LED_PORT |= ( LED );
				POWER_SW_PORT &= ~( POWER_SW );
				//    HDD
				tmp = 1;
			  }
			  
			
			//        HDD
			if ( TimerCount > TIME_KEY_AHDDON )
			  {
				if ( Mode == MODE_AS )
				  {
				    writeE(   MODE_START_ADR, 0);
				    writeE( MODE_START_ADR+1, 0);
					Mode = 0;
				  }
				  
				else 
				  if ( Mode == 0 )
				    {
				      writeE(   MODE_START_ADR, MODE_AS_BYTE1);
				      writeE( MODE_START_ADR+1, MODE_AS_BYTE2);
					  Mode = MODE_AS;
				    }
			  }
		  }
			if ( Mode == MODE_AS )
		      {
		        LED_PORT |= ( LED );
			    Delay( DELAY_LED_OFF );
				Delay( DELAY_LED_OFF );
			    LED_PORT &= ~( LED );
			    Delay( DELAY_LED_ON_SM );
		      }	
	        if ( tmp == 1 )
			  {
 		        LED_PORT |= ( LED );
			    Delay( DELAY_LED_OFF );
				Delay( DELAY_LED_OFF );
			    LED_PORT &= ~( LED );
			    Delay( DELAY_LED_ON_SM );
				LED_PORT |= ( LED );
				Delay( DELAY_LED_OFF_SM );
				LED_PORT &= ~( LED );
			    Delay( DELAY_LED_ON_SM );
			  }		  
			  		
      }  
} 

